@@ -1,5 +1,5 @@
 # rawdog: RSS aggregator without delusions of grandeur.
-# Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013, 2014 Adam Sampson <ats@offog.org>
+# Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013, 2014, 2015, 2016 Adam Sampson <ats@offog.org>
 #
 # rawdog is free software; you can redistribute and/or modify it
 # under the terms of that license as published by the Free Software
@@ -16,20 +16,32 @@
 # Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301, USA, or see http://www.gnu.org/.
 
-VERSION = "2.20"
+VERSION = "2.22"
 HTTP_AGENT = "rawdog/" + VERSION
 STATE_VERSION = 2
 
-import feedparser, plugins
-from persister import Persistable, Persister
-import os, time, getopt, sys, re, cgi, socket, urllib2, calendar
-import string, locale
-from StringIO import StringIO
-import types
-import threading
-import hashlib
+import rawdoglib.feedscanner
+from rawdoglib.persister import Persistable, Persister
+from rawdoglib.plugins import Box, call_hook, load_plugins
+
+from cStringIO import StringIO
 import base64
-import feedscanner
+import calendar
+import cgi
+import feedparser
+import getopt
+import hashlib
+import locale
+import os
+import re
+import socket
+import string
+import sys
+import threading
+import time
+import types
+import urllib2
+import urlparse
 
 try:
 	import tidylib
@@ -62,12 +74,18 @@ def get_system_encoding():
 def safe_ftime(format, t):
 	"""Format a time value into a string in the current locale (as
 	time.strftime), but encode the result as ASCII HTML."""
-	u = unicode(time.strftime(format, t), get_system_encoding())
+	try:
+		u = unicode(time.strftime(format, t), get_system_encoding())
+	except ValueError, e:
+		u = u"(bad time %s; %s)" % (repr(t), str(e))
 	return encode_references(u)
 
 def format_time(secs, config):
 	"""Format a time and date nicely."""
-	t = time.localtime(secs)
+	try:
+		t = time.localtime(secs)
+	except ValueError, e:
+		return u"(bad time %s; %s)" % (repr(secs), str(e))
 	format = config["datetimeformat"]
 	if format is None:
 		format = config["timeformat"] + ", " + config["dayformat"]
@@ -116,13 +134,17 @@ def sanitise_html(html, baseurl, inline, config):
 			html = "<p>" + html
 
 	if config["tidyhtml"]:
-		args = {"numeric_entities": 1,
-		        "output_html": 1,
-		        "output_xhtml": 0,
-		        "output_xml": 0,
-		        "wrap": 0}
-		plugins.call_hook("mxtidy_args", config, args, baseurl, inline)
-		plugins.call_hook("tidy_args", config, args, baseurl, inline)
+		args = {
+			"numeric_entities": 1,
+			"input_encoding": "ascii",
+			"output_encoding": "ascii",
+			"output_html": 1,
+			"output_xhtml": 0,
+			"output_xml": 0,
+			"wrap": 0,
+			}
+		call_hook("mxtidy_args", config, args, baseurl, inline)
+		call_hook("tidy_args", config, args, baseurl, inline)
 		if tidylib is not None:
 			# Disable PyTidyLib's somewhat unhelpful defaults.
 			tidylib.BASE_OPTIONS = {}
@@ -136,16 +158,18 @@ def sanitise_html(html, baseurl, inline, config):
 		              : output.rfind("</body>")].strip()
 
 	html = html.decode("UTF-8")
-	box = plugins.Box(html)
-	plugins.call_hook("clean_html", config, box, baseurl, inline)
+	box = Box(html)
+	call_hook("clean_html", config, box, baseurl, inline)
 	return box.value
 
 def select_detail(details):
 	"""Pick the preferred type of detail from a list of details. (If the
 	argument isn't a list, treat it as a list of one.)"""
-	TYPES = {"text/html": 30,
-	         "application/xhtml+xml": 20,
-	         "text/plain": 10}
+	TYPES = {
+		"text/html": 30,
+		"application/xhtml+xml": 20,
+		"text/plain": 10,
+		}
 
 	if details is None:
 		return None
@@ -228,8 +252,8 @@ def fill_template(template, bits):
 	including sections bracketed by __if_x__ .. [__else__ ..]
 	__endif__ if bits["x"] is not "". If not bits.has_key("x"),
 	__x__ expands to ""."""
-	result = plugins.Box()
-	plugins.call_hook("fill_template", template, bits, result)
+	result = Box()
+	call_hook("fill_template", template, bits, result)
 	if result.value is not None:
 		return result.value
 
@@ -342,7 +366,7 @@ def is_timeout_exception(exc):
 	#   <urlopen error _ssl.c:489: The handshake operation timed out>
 	# Debian python 2.7.8-1:
 	#   <urlopen error ('_ssl.c:563: The handshake operation timed out',)>
-	return (timeout_re.search(str(exc)) is not None)
+	return timeout_re.search(str(exc)) is not None
 
 class BasicAuthProcessor(urllib2.BaseHandler):
 	"""urllib2 handler that does HTTP basic authentication
@@ -416,7 +440,7 @@ class Feed:
 	def needs_update(self, now):
 		"""Return True if it's time to update this feed, or False if
 		its update period has not yet elapsed."""
-		return ((now - self.last_update) >= self.period)
+		return (now - self.last_update) >= self.period
 
 	def get_state_filename(self):
 		return "feeds/%s.state" % (short_hash(self.url),)
@@ -449,7 +473,7 @@ class Feed:
 			# being published by the feed, we have to turn it off.
 			handlers.append(DisableIMProcessor())
 
-		plugins.call_hook("add_urllib2_handlers", rawdog, config, self, handlers)
+		call_hook("add_urllib2_handlers", rawdog, config, self, handlers)
 
 		url = self.url
 		# Turn plain filenames into file: URLs. (feedparser will open
@@ -517,7 +541,22 @@ class Feed:
 				i += 1
 			location = responses[i - 1].get("location")
 
-			if location is None:
+			# According to RFC 2616, the Location header should be
+			# an absolute URI. This doesn't stop the occasional
+			# server sending something like "Location: /" or
+			# "Location: //foo/bar". It's usually a sign of
+			# brokenness, so fail rather than trying to interpret
+			# it liberally.
+			valid_uri = True
+			if location is not None:
+				parsed = urlparse.urlparse(location)
+				if parsed.scheme == "" or parsed.netloc == "":
+					valid_uri = False
+
+			if not valid_uri:
+				errors.append("New URL:     " + location)
+				errors.append("The feed returned a permanent redirect, but with an invalid new location.")
+			elif location is None:
 				errors.append("The feed returned a permanent redirect, but without a new location.")
 			else:
 				errors.append("New URL:     " + location)
@@ -573,7 +612,7 @@ class Feed:
 			fatal = True
 
 		old_error = "\n".join(errors)
-		plugins.call_hook("feed_fetched", rawdog, config, self, p, old_error, not fatal)
+		call_hook("feed_fetched", rawdog, config, self, p, old_error, not fatal)
 
 		if len(errors) != 0:
 			print >>sys.stderr, "Feed:        " + old_url
@@ -612,8 +651,8 @@ class Feed:
 		sequence = 0
 		for entry_info in p["entries"]:
 			article = Article(feed, entry_info, now, sequence)
-			ignore = plugins.Box(False)
-			plugins.call_hook("article_seen", rawdog, config, article, ignore)
+			ignore = Box(False)
+			call_hook("article_seen", rawdog, config, article, ignore)
 			if ignore.value:
 				continue
 			seen_articles.add(article.hash)
@@ -629,10 +668,10 @@ class Feed:
 
 			if existing_article is not None:
 				existing_article.update_from(article, now)
-				plugins.call_hook("article_updated", rawdog, config, existing_article, now)
+				call_hook("article_updated", rawdog, config, existing_article, now)
 			else:
 				articles[article.hash] = article
-				plugins.call_hook("article_added", rawdog, config, article, now)
+				call_hook("article_added", rawdog, config, article, now)
 
 		if config["currentonly"]:
 			for (hash, a) in articles.items():
@@ -727,7 +766,7 @@ class Article:
 		self.last_seen = now
 
 	def can_expire(self, now, config):
-		return ((now - self.last_seen) > config["expireage"])
+		return (now - self.last_seen) > config["expireage"]
 
 	def get_sort_date(self, config):
 		if config["sortbyfeeddate"]:
@@ -757,7 +796,11 @@ class DayWriter:
 		self.counter += 1
 
 	def time(self, s):
-		tm = time.localtime(s)
+		try:
+			tm = time.localtime(s)
+		except ValueError:
+			# e.g. "timestamp out of range for platform time_t"
+			return
 		if tm[:3] != self.lasttime[:3] and self.config["daysections"]:
 			self.close(0)
 			self.start_day(tm)
@@ -779,7 +822,13 @@ def parse_time(value, default="m"):
 	in seconds. If no unit is specified, use minutes by default; specify
 	the default argument to change this. Raises ValueError if the format
 	isn't recognised."""
-	units = { "s" : 1, "m" : 60, "h" : 3600, "d" : 86400, "w" : 604800 }
+	units = {
+		"s": 1,
+		"m": 60,
+		"h": 3600,
+		"d": 86400,
+		"w": 604800,
+		}
 	for unit, size in units.items():
 		if value.endswith(unit):
 			return int(value[:-len(unit)]) * size
@@ -823,7 +872,8 @@ def parse_feed_args(argparams, arglines):
 			args[name] = parse_time(value)
 	return args
 
-class ConfigError(Exception): pass
+class ConfigError(Exception):
+	pass
 
 class Config:
 	"""The aggregator's configuration."""
@@ -879,9 +929,14 @@ class Config:
 			"useids": False,
 			}
 
-	def __getitem__(self, key): return self.config[key]
-	def get(self, key, default=None): return self.config.get(key, default)
-	def __setitem__(self, key, value): self.config[key] = value
+	def __getitem__(self, key):
+		return self.config[key]
+
+	def get(self, key, default=None):
+		return self.config.get(key, default)
+
+	def __setitem__(self, key, value):
+		self.config[key] = value
 
 	def reload(self):
 		self.log("Reloading config files")
@@ -953,7 +1008,7 @@ class Config:
 			self["defines"][l[0]] = l[1]
 		elif l[0] == "plugindirs":
 			for dir in parse_list(l[1]):
-				plugins.load_plugins(dir, self)
+				load_plugins(dir, self)
 		elif l[0] == "outputfile":
 			self["outputfile"] = l[1]
 		elif l[0] == "maxarticles":
@@ -1016,9 +1071,9 @@ class Config:
 			self["useids"] = parse_bool(l[1])
 		elif l[0] == "include":
 			self.load(l[1], False)
-		elif plugins.call_hook("config_option_arglines", self, l[0], l[1], arglines):
+		elif call_hook("config_option_arglines", self, l[0], l[1], arglines):
 			handled_arglines = True
-		elif plugins.call_hook("config_option", self, l[0], l[1]):
+		elif call_hook("config_option", self, l[0], l[1]):
 			pass
 		else:
 			raise ConfigError("Unknown config command: " + l[0])
@@ -1069,7 +1124,7 @@ class AddFeedEditor:
 
 def add_feed(filename, url, rawdog, config):
 	"""Try to add a feed to the config file."""
-	feeds = feedscanner.feeds(url)
+	feeds = rawdoglib.feedscanner.feeds(url)
 	if feeds == []:
 		print >>sys.stderr, "Cannot find any feeds in " + url
 		return
@@ -1148,7 +1203,7 @@ class FeedFetcher:
 
 			config.log("[", num, "] Fetching feed: ", job)
 			feed = rawdog.feeds[job]
-			plugins.call_hook("pre_update_feed", rawdog, config, feed)
+			call_hook("pre_update_feed", rawdog, config, feed)
 			result = feed.fetch(rawdog, config)
 
 			with self.lock:
@@ -1382,18 +1437,18 @@ class Rawdog(Persistable):
 				    and self.feeds.has_key(url)
 				    and article.can_expire(now, config)
 				    and feedcounts[url] > self.feeds[url].get_keepmin(config)):
-					plugins.call_hook("article_expired", self, config, article, now)
+					call_hook("article_expired", self, config, article, now)
 					count += 1
 					feedcounts[url] -= 1
 					del articles[key]
 			config.log("Expired ", count, " articles, leaving ", len(articles))
 
-			return (count > 0)
+			return count > 0
 
 		count = 0
 		for url in update_feeds:
 			count += 1
-			config.log("Updating feed ", count, " of " , numfeeds, ": ", url)
+			config.log("Updating feed ", count, " of ", numfeeds, ": ", url)
 			feed = self.feeds[url]
 
 			if config["splitstate"]:
@@ -1404,10 +1459,10 @@ class Rawdog(Persistable):
 				articles = self.articles
 
 			content = fetched[url]
-			plugins.call_hook("mid_update_feed", self, config, feed, content)
+			call_hook("mid_update_feed", self, config, feed, content)
 			rc = feed.update(self, now, config, articles, content)
 			url = feed.url
-			plugins.call_hook("post_update_feed", self, config, feed, rc)
+			call_hook("post_update_feed", self, config, feed, rc)
 			if rc:
 				seen_some_items.add(url)
 				if config["splitstate"]:
@@ -1508,7 +1563,7 @@ __feeditems__
 		try:
 			print self.get_template(config, name),
 		except KeyError:
-			print >>sys.stderr, "Unknown template name: " + a
+			print >>sys.stderr, "Unknown template name: " + name
 
 	def write_article(self, f, article, config):
 		"""Write an article to the given file."""
@@ -1581,7 +1636,7 @@ __feeditems__
 		else:
 			itembits["date"] = ""
 
-		plugins.call_hook("output_item_bits", self, config, feed, article, itembits)
+		call_hook("output_item_bits", self, config, feed, article, itembits)
 		itemtemplate = self.get_template(config, "item")
 		f.write(fill_template(itemtemplate, itembits))
 
@@ -1666,14 +1721,12 @@ __feeditems__
 	def get_main_template_bits(self, config):
 		"""Get the bits that are used in the default main template,
 		with the exception of items and num_items."""
-		bits = { "version" : VERSION }
+		bits = {"version": VERSION}
 		bits.update(config["defines"])
 
-		refresh = config["expireage"]
-		for feed in self.feeds.values():
-			if feed.period < refresh: refresh = feed.period
-
-		bits["refresh"] = """<meta http-equiv="Refresh" """ + 'content="' + str(refresh) + '"' + """>"""
+		refresh = min([config["expireage"]]
+		              + [feed.period for feed in self.feeds.values()])
+		bits["refresh"] = '<meta http-equiv="Refresh" content="' + str(refresh) + '">'
 
 		f = StringIO()
 		self.write_feedlist(f, config)
@@ -1687,22 +1740,22 @@ __feeditems__
 		"""Write a regular rawdog HTML output file."""
 		f = StringIO()
 		dw = DayWriter(f, config)
-		plugins.call_hook("output_items_begin", self, config, f)
+		call_hook("output_items_begin", self, config, f)
 
 		for article in articles:
-			if not plugins.call_hook("output_items_heading", self, config, f, article, article_dates[article]):
+			if not call_hook("output_items_heading", self, config, f, article, article_dates[article]):
 				dw.time(article_dates[article])
 
 			self.write_article(f, article, config)
 
 		dw.close()
-		plugins.call_hook("output_items_end", self, config, f)
+		call_hook("output_items_end", self, config, f)
 
 		bits = self.get_main_template_bits(config)
 		bits["items"] = f.getvalue()
 		f.close()
 		bits["num_items"] = str(len(articles))
-		plugins.call_hook("output_bits", self, config, bits)
+		call_hook("output_bits", self, config, bits)
 		s = fill_template(self.get_template(config, "page"), bits)
 		outputfile = config["outputfile"]
 		if outputfile == "-":
@@ -1731,7 +1784,7 @@ __feeditems__
 			article_list = list_articles(self.articles)
 		numarticles = len(article_list)
 
-		if not plugins.call_hook("output_sort_articles", self, config, article_list):
+		if not call_hook("output_sort_articles", self, config, article_list):
 			article_list.sort()
 
 		if config["maxarticles"] != 0:
@@ -1766,16 +1819,16 @@ __feeditems__
 				articles.append(a)
 				article_dates[a] = -date
 
-		plugins.call_hook("output_write", self, config, articles)
+		call_hook("output_write", self, config, articles)
 
-		if not plugins.call_hook("output_sorted_filter", self, config, articles):
+		if not call_hook("output_sorted_filter", self, config, articles):
 			(articles, dup_count) = self.write_remove_dups(articles, config, now)
 		else:
 			dup_count = 0
 
 		config.log("Selected ", len(articles), " of ", numarticles, " articles to write; ignored ", dup_count, " duplicates")
 
-		if not plugins.call_hook("output_write_files", self, config, articles, article_dates):
+		if not call_hook("output_write_files", self, config, articles, article_dates):
 			self.write_output_file(articles, article_dates, config)
 
 		config.log("Finished write")
@@ -1899,7 +1952,10 @@ def main(argv):
 			return 1
 		if verbose:
 			config["verbose"] = True
-	load_config("config")
+		return 0
+	rc = load_config("config")
+	if rc != 0:
+		return rc
 
 	global persister
 	persister = Persister(config)
@@ -1916,7 +1972,7 @@ def main(argv):
 
 	rawdog.sync_from_config(config)
 
-	plugins.call_hook("startup", rawdog, config)
+	call_hook("startup", rawdog, config)
 
 	for o, a in optlist:
 		if o in ("-a", "--add"):
@@ -1924,7 +1980,9 @@ def main(argv):
 			config.reload()
 			rawdog.sync_from_config(config)
 		elif o in ("-c", "--config"):
-			load_config(a)
+			rc = load_config(a)
+			if rc != 0:
+				return rc
 			rawdog.sync_from_config(config)
 		elif o in ("-f", "--update-feed"):
 			rawdog.update(config, a)
@@ -1945,9 +2003,8 @@ def main(argv):
 		elif o in ("-w", "--write"):
 			rawdog.write(config)
 
-	plugins.call_hook("shutdown", rawdog, config)
+	call_hook("shutdown", rawdog, config)
 
 	rawdog_p.close()
 
 	return 0
-
